// Iteration.java                                   MM 2013

/**
 * Nullstelleniteration mittels Newton und Regula falsi.
 * @author Monika Meiler
 */
public class Iteration
{
/* ----------------------------------------------------- */
                                                // Attribut
/**
 * Aktuelle Iterationstiefe.
 */
  private int anzahl = 0;

/* ----------------------------------------------------- */
                                             // get-Methode
/**
 * Liest Iterationstiefe.
 * @return anzahl
 */
  public int getAnzahl()
  {
    return anzahl;
  }

/* ----------------------------------------------------- */
                                               // Verfahren
/**
 * Nullstellenbestimmung mit Newton.
 * @param fkt differenzierbare Funktion
 * @param x0 Naeherung
 * @param eps Genauigkeit
 * @param max maximale Iterationstiefe
 * @return Naeherung einer Nullstelle,
 *         falls anzahl == max
 *         wurde die gewuenschte Genauigkeit nicht erreicht.
 */
  public double newton( DifferenzierbareFunktion fkt,
    double x0, double eps, int max)
  {
    double y0 = fkt.wert( x0), y1;
    anzahl = 0;
                                // Naeherung ist Nullstelle
    if( Math.abs( y0) < eps) return x0;

    do                                            // Newton
    {
      // System.out.println( anzahl + ": " + x0);

      y1 = fkt.wertErsteAbleitung( x0);   // neue Naeherung
      x0 -= y0 / y1;
      y0 = fkt.wert( x0);

/*
      if( y1 == 0) anzahl = max;     // Division durch Null
      else anzahl++;
*/
      if( y1 == 0) return x0;        // Division durch Null
      anzahl++;
                                                 // Abbruch
    } while(( Math.abs( y0) >= eps) && ( anzahl < max));

    return x0;
  }

/**
 * Nullstellenbestimmung mit Newton
 * fuer Wertberechnung einer Funktion
 * mittels ihrer Umkehrfunktion.
 * @param fkt differenzierbare Umkehrfunktion
 * @param arg Argument, wird als 1. Naeherung verwendet
 * @param eps Genauigkeit
 * @param max maximale Iterationstiefe
 * @return Naeherung einer Nullstelle,
 *         falls anzahl == max
 *         wurde die gewuenschte Genauigkeit nicht erreicht.
 */
  public double newton_1( DifferenzierbareFunktion fkt,
    double arg, double eps, int max)
  {
    double x0 = arg, y0 = fkt.wert( x0) - arg, y1;
    anzahl = 0;
                                // Naeherung ist Nullstelle
    if( Math.abs( y0) < eps) return x0;

    do                                            // Newton
    {
      // System.out.println( anzahl + ": " + x0);

      y1 = fkt.wertErsteAbleitung( x0);   // neue Naeherung
      x0 -= y0 / y1;
      y0 = fkt.wert( x0) - arg;

      if( y1 == 0) return x0;        // Division durch Null
      anzahl++;
                                                 // Abbruch
    } while(( Math.abs( y0) >= eps)&& ( anzahl < max));

    return x0;
  }

/**
 * Nullstellenbestimmung mit Ragula falsi.
 * @param fkt Funktion
 * @param x0 1. Naeherung (sgn( f( x0)) != sgn( f( x1)))
 * @param x1 2. Naeherung (sgn( f( x0)) != sgn( f( x1)))
 * @param eps Genauigkeit
 * @param max maximale Iterationstiefe
 * @return Naeherung einer Nullstelle,
 *         falls anzahl == max
 *         wurde die gewuenschte Genauigkeit nicht erreicht.
 */
  public double regulaFalsi( Funktion fkt,
    double x0, double x1, double eps, int max)
  {
    double y0 = fkt.wert( x0);
    double y1 = fkt.wert( x1);
                                          // Vorzeichentest
    if( Math.signum( y0) == Math.signum( y1))
    {
      anzahl = max;
      return x0;
    }

    anzahl = 0;
                              // Naeherungen ist Nullstelle
    if( Math.abs( y0) < eps) return x0;
    if( Math.abs( y1) < eps) return x1;

    do                                      // Regula falsi
    {
      // System.out.print  ( " x0: " + x0);
      // System.out.println( "\tx1: " + x1);
                                          // neue Naeherung
      double x = x0 - ( x1 - x0) / ( y1 - y0) * y0;
      if( y1 == y0) return x;        // Division durch Null

      double y = fkt.wert( x);
                                              // Vorzeichen
      if( Math.signum( y) == Math.signum( y1))
      {
        x1 = x0;
        y1 = y0;
      }
      x0 = x;
      y0 = y;

      anzahl++;                                  // Abbruch
    } while(( Math.abs( y0) >= eps) && ( anzahl < max));

    return x0;
  }
}