// Polynom.java                                     MM 2013
import Tools.IO.*;                              // Eingaben

/**
 * Ganze rationale Funktion (Polynom r-ten Grades),
 * f(x) = ar*x^r + ... a2*x^2 + a1*x + a0, D = W = R.
 * @author Monika Meiler
 */
public class Polynom extends DifferenzierbareFunktion
{
/* ----------------------------------------------------- */
                                               // Attribute
/**
 * Polynomkoeffizienten.
 */
  private double[] koeffizienten = null;

/* ----------------------------------------------------- */
                                            // set-Methoden
/**
 * Setzt Polynomkoeffizienten fest.
 * @param koeff Koeffizienten des Polynoms
 * @return true, bei erfolgreichem Eintrag
 *         false, falls keine Koeffizienten existieren
 */
  public boolean setPolynom( double[] koeff)
  {
    if( koeff == null || koeff.length == 0)
    {
      koeffizienten = null;
      return false;
    }
    koeffizienten = koeff;
    return true;
  }

/**
 * Polynomeingabe ueber Konsole.
 * @return true, bei erfolgreichem Eintrag
 */
  public boolean konsolenEingabe()
  {
// Eingabe der Polynomkoeffizienten
    System.out.println( "Polynomeingabe");
    int grad =
    IOTools.readInteger( " Grad des Polynoms (>=0): ");

    double[] koeff = null;
    if( grad >= 0)
    {
      koeff = new double[ grad + 1];
      for( int i = 0; i < grad + 1; i++)
        koeff[ i] =
        IOTools.readDouble( " " + i + ". Koeffizient: ");
    }

// Setzen der Polynomkoeffizienten
    return setPolynom( koeff);
  }

/* ------------------------------------------------------*/
                                        // service-Methoden
/**
 * Berechnen eines Funktionswertes nach HORNER.
 * @param arg Argument
 * @return f( arg)
 */
  public double wert( double arg)
  {
    double erg = 0;
    for( int i = koeffizienten.length - 1; i >= 0; i--)
      erg = erg * arg + koeffizienten[ i];
    return erg;
  }

/**
 * Berechnen einer Ableitung nach erweitertem HORNER.
 * @param arg Argument
 * @return f'( arg)
 */
  public double wertErsteAbleitung( double arg)
  {
    double erg = 0, erg1 = 0;
    for( int i = koeffizienten.length - 1; i > 0; i--)
    {
      erg = erg * arg + koeffizienten[ i];
      erg1 = erg1 * arg + erg;
    }
    return erg1;
  }

/* ----------------------------------------------------- */
                                        // toString-Methode
/**
 * Darstellung eines Polynoms.
 * @return Funktion in linearer Schreibweise
 */
  public String toString()
  {
    String str = "" + koeffizienten[ 0];
    for( int i = 1; i < koeffizienten.length; i++)
      str += " + " + koeffizienten[ i] + " x^" + i;
    return str;
  }
}
