// Sinus.java                                       MM 2013

/**
 * Sinusfunktion f(x) = sin( x), D = R, W = [ 1, -1].
 * @author Monika Meiler
 */
public class Sinus extends DifferenzierbareFunktion
{
/* ----------------------------------------------------- */
                                        // service-Methoden
/**
 * Berechnen eines Funktionswertes.
 * @param arg Argument
 * @return sin( arg)
 */
  public double wert( double arg)
  {
// einfache Reihenentwicklung
//  return mySin( arg);
// verbesserte Reihenentwicklung
    return mySinBesser( arg);
  }

/**
 * Berechnen eines Funktionswertes durch Reihenentwicklung:
 * sin x = x - ( x ^ 3) / 3! + ( x ^ 5) / 5! ...
 * @param arg Argument
 * @return sin( arg)
 */
  private double mySin( double arg)
  {
    double x = arg;
    double alteSumme, neueSumme = x, summand = x;
    int j = 1;

    do                                 // Reihenentwicklung
    {
      summand *= -x * x / ++j;             // neuer Summand
      summand /= ++j;

      alteSumme = neueSumme;                  // neue Summe
      neueSumme += summand;
    } while(( neueSumme != alteSumme) && ( j < 1000));

    return alteSumme;
  }

/**
 * Verbesserte Sinusfunktion:
 * Transformationdes Arguments in das Intervall [0,PI/2].
 * @param arg Argument
 * @return sin( arg) verbessert
 */
  private double mySinBesser( double arg)
  {
    double x = arg;
    int s = 1;                                // Vorzeichen
// x < 0
    if( x < 0)
    {
      x = -x;
      s = -s;
    }
// x > PI
    if( x > Math.PI)
    {
      int temp = ( int)( x / Math.PI);
      if( temp % 2 != 0) s = -s;       // Vorzeichenwechsel
      x = x - temp * Math.PI;
    }
// x > PI/2
    if( x > Math.PI / 2) x = Math.PI - x;

// Trivialfaelle
    if( x == 0 ) return 0;
    if( x == Math.PI / 2) return s;

// Reihenentwicklung
    return s * mySin( x);
  }

/**
 * Berechnen der ersten Ableitung
 * f'( x) = cos( x) = sin( PI/2 + x).
 * @param arg Argument
 * @return f'( arg)
 */
  public double wertErsteAbleitung( double arg)
  {
    return wert( Math.PI / 2 + arg);
  }

/* ----------------------------------------------------- */
                                        // toString-Methode
/**
 * Darstellen der Sinusfunktion.
 * @return Funktion in linearer Schreibweise
 */
  public String toString()
  {
    return "sin( x)";
  }
}